<?php
/**
 * Plugin Name: Bayarcash for Fluent Forms
 * Plugin URI: https://bayarcash.com
 * Description: Bayarcash is a local Malaysia payment gateway that support FPX, Direct Debit, DuitNow QR, DuitNow Online Banking/Wallets, BNPL (SPayLater & Boost PayFlex) as well cross-border payment (Indonesia, Singapore & Thailand).
 * Version: 1.2.0
 * Author: Web Impian
 * Author URI: https://webimpian.com
 * License: GNU General Public License v3.0
 * License URI: https://www.gnu.org/licenses/gpl-3.0.txt
 * Text Domain: bayarcash-for-fluent-forms
 * Requires PHP: 7.4
 * Requires at least: 5.0
 * Tested up to: 6.7
 */

if (!defined('ABSPATH')) {
	die;
}

const FF_BAYARCASH_MODULE_VERSION = 'v1.0.0';

// Load Composer's autoloader
$composer_autoloader = plugin_dir_path(__FILE__) . 'vendor/autoload.php';
if (!file_exists($composer_autoloader)) {
	add_action('admin_notices', function() {
		?>
        <div class="notice notice-error">
            <p><?php _e('Bayarcash for Fluent Forms: Required dependencies are missing. Please run "composer install" in the plugin directory.', 'bayarcash-for-fluent-forms'); ?></p>
        </div>
		<?php
	});
	return;
}
require_once $composer_autoloader;

// Verify Bayarcash SDK is loaded
if (!class_exists('\Webimpian\BayarcashSdk\Bayarcash')) {
	add_action('admin_notices', function() {
		?>
        <div class="notice notice-error">
            <p><?php _e('Bayarcash for Fluent Forms: Bayarcash SDK could not be loaded. Please check your installation.', 'bayarcash-for-fluent-forms'); ?></p>
        </div>
		<?php
	});
	return;
}

class Bayarcash_Fluent_Forms {
	private static $_instance;

	public static function get_instance(): Bayarcash_Fluent_Forms {
		if (self::$_instance == null) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function __construct() {
		$this->define();
		$this->includes();
		$this->add_filters();
		$this->add_scripts();
	}

	private function define(): void {
		define('FF_BAYARCASH_FILE', __FILE__);
		define('FF_BAYARCASH_BASENAME', plugin_basename(FF_BAYARCASH_FILE));
		define('FF_BAYARCASH_FSLUG', 'fluent_form_bayarcash');
	}

	private function includes(): void {
		$includes_dir = plugin_dir_path(FF_BAYARCASH_FILE) . 'includes/';

		//Framework files
		require_once $includes_dir . 'codestar-framework/classes/setup.class.php';

		// Admin-only includes
		if (is_admin()) {
			require_once $includes_dir . 'admin/global-settings.php';
			require_once $includes_dir . 'admin/form-settings.php';
			require_once $includes_dir . 'admin/gateway-fee.php';
			require_once $includes_dir . 'admin/backup-settings.php';
		}

		// Payment processing classes
		require_once $includes_dir . 'class-register.php';
		require_once $includes_dir . 'class-purchase.php';
		require_once $includes_dir . 'add-gateway-fee.php';
	}

	private function add_filters(): void {
		add_filter('plugin_action_links_' . FF_BAYARCASH_BASENAME, function($links) {
			$settings_link = sprintf(
				'<a href="%1$s">%2$s</a>',
				admin_url('admin.php?page=bayarcash-for-fluent-forms'),
				esc_html__('Settings', 'bayarcash-for-fluent-forms')
			);
			return array_merge(['settings' => $settings_link], $links);
		});
	}

	private function add_scripts(): void {
		add_action('wp_enqueue_scripts', function() {
			$js_url = plugins_url('includes/js/bayarcash-fluent-forms.js', FF_BAYARCASH_FILE);

			try {
				wp_enqueue_script(
					'bayarcash-fluent-forms',
					$js_url,
					['jquery'],
					FF_BAYARCASH_MODULE_VERSION,
					true
				);

				// Get fee settings from CodeStar Framework
				$fee_settings = get_option('fluent_form_bayarcash', []);

				// Transform the settings to match JavaScript expectations
				$transformed_settings = [];
				$payment_methods = ['fpx_banking', 'credit_card', 'duitnow_banking', 'duitnow_qr'];

				foreach ($payment_methods as $method) {
					$transformed_settings[$method] = [
						"{$method}_fee_enable" => !empty($fee_settings["{$method}_fee_enable"]) ? 'true' : 'false',
						"{$method}_fee_type" => $fee_settings["{$method}_fee_type"] ?? 'flat',
						"{$method}_flat_fee" => $fee_settings["{$method}_flat_fee"] ?? '0',
						"{$method}_percentage_fee" => $fee_settings["{$method}_percentage_fee"] ?? '0',
						"{$method}_fee_description" => $fee_settings["{$method}_fee_description"] ?? ''
					];
				}

				//error_log('Bayarcash FF Settings: ' . print_r($transformed_settings, true));

				// Add localized data
				wp_localize_script('bayarcash-fluent-forms', 'bayarcashFF', [
					'ajaxUrl' => admin_url('admin-ajax.php'),
					'nonce' => wp_create_nonce('bayarcash_ff_nonce'),
					'feeSettings' => $transformed_settings
				]);

			} catch (Exception $e) {
				error_log('Bayarcash FF: Error enqueueing JavaScript: ' . $e->getMessage());
			}
		});
	}

}


// Initialize the plugin
add_action('plugins_loaded', function() {
	// Check for required dependencies
	if (!class_exists('FluentFormPro\Payments\PaymentHelper') ||
	    !class_exists('FluentFormPro\Payments\PaymentMethods\BaseProcessor')) {
		add_action('admin_notices', function() {
			?>
			<div class="notice notice-error">
				<p><?php _e('Bayarcash for Fluent Forms requires Fluent Forms Pro to be installed and activated.', 'bayarcash-for-fluent-forms'); ?></p>
			</div>
			<?php
		});
		return;
	}

	Bayarcash_Fluent_Forms::get_instance();
});