<?php

$slug = FF_BAYARCASH_FSLUG;

$ff_module_settings = get_option( '__fluentform_payment_module_settings' );

// Ensure we have a default array structure even if the option doesn't exist
$ff_module_settings = is_array($ff_module_settings) ? $ff_module_settings : array();
$current_currency = $ff_module_settings['currency'] ?? '';

// Get the plugin directory URL for the logo path
$plugin_url = plugin_dir_url( dirname( __FILE__, 2 ) );

function verify_bayarcash_token($token, $is_sandbox = false): array {
	if (empty($token)) {
		return array(
			'status' => 'error',
			'message' => __('Token cannot be empty', 'bayarcash-for-fluent-forms')
		);
	}

	$api_url = $is_sandbox
		? 'https://console.bayarcash-sandbox.com/api/v2/portals'
		: 'https://console.bayar.cash/api/v2/portals';

	$response = wp_remote_get($api_url, array(
		'headers' => array(
			'Authorization' => 'Bearer ' . $token,
			'Accept' => 'application/json'
		),
		'timeout' => 15
	));

	if (is_wp_error($response)) {
		return array(
			'status' => 'error',
			'message' => __('Connection error: ', 'bayarcash-for-fluent-forms') . $response->get_error_message()
		);
	}

	$body = wp_remote_retrieve_body($response);
	$data = json_decode($body, true);

	if (!isset($data['data'])) {
		return array(
			'status' => 'error',
			'message' => __('Invalid token or API response', 'bayarcash-for-fluent-forms')
		);
	}

	return array(
		'status' => 'success',
		'message' => __('Token verified successfully', 'bayarcash-for-fluent-forms')
	);
}

function get_bayarcash_portals(): array {
	// Get current settings
	$settings = get_option(FF_BAYARCASH_FSLUG);
	$is_sandbox = $settings['enable-sandbox'] ?? false;
	$token = $settings['personal-access-token'] ?? '';

	if (empty($token)) {
		return array('' => __('Please enter Personal Access Token first', 'bayarcash-for-fluent-forms'));
	}

	$api_url = $is_sandbox
		? 'https://console.bayarcash-sandbox.com/api/v2/portals'
		: 'https://console.bayar.cash/api/v2/portals';

	$response = wp_remote_get($api_url, array(
		'headers' => array(
			'Authorization' => 'Bearer ' . $token,
			'Accept' => 'application/json'
		),
		'timeout' => 15
	));

	if (is_wp_error($response)) {
		return array('' => __('Error fetching portals: ' . $response->get_error_message(), 'bayarcash-for-fluent-forms'));
	}

	$body = wp_remote_retrieve_body($response);
	$data = json_decode($body, true);

	if (!isset($data['data']) || !is_array($data['data'])) {
		return array('' => __('Invalid API response', 'bayarcash-for-fluent-forms'));
	}

	$portals = array('' => __('Select a Portal', 'bayarcash-for-fluent-forms'));
	foreach ($data['data'] as $portal) {
		$portals[$portal['portal_key']] = $portal['portal_name'];
	}

	return $portals;
}

// Add AJAX handlers
add_action('wp_ajax_save_bayarcash_settings', function() {
	check_ajax_referer('bayarcash_settings', 'nonce');

	parse_str($_POST['settings'], $settings);
	update_option(FF_BAYARCASH_FSLUG, $settings[FF_BAYARCASH_FSLUG]);

	wp_send_json_success();
});

add_action('wp_ajax_verify_bayarcash_token', function() {
	check_ajax_referer('bayarcash_settings', 'nonce');

	$settings = get_option(FF_BAYARCASH_FSLUG);
	$token = $_POST['token'] ?? '';
	$is_sandbox = filter_var($_POST['is_sandbox'] ?? false, FILTER_VALIDATE_BOOLEAN);

	$result = verify_bayarcash_token($token, $is_sandbox);
	wp_send_json($result);
});

// Combine all fields into one section
$global_configuration_fields = array(
	array(
		'type' => 'notice',
		'style' => 'danger',
		'content' => sprintf( __( 'The default currency is set to non compatible currencies! %sClick here%s to update currency configuration.', 'bayarcash-for-fluent-forms' ), '<a target=_blank href=' . admin_url('admin.php?page=fluent_forms_settings&component=payment_settings#/') . ' >', '</a>' ),
		'class' => $current_currency == 'MYR' ? 'hidden' : '',
	),
	array(
		'type'    => 'subheading',
		'content' => 'Credentials',
	),
	array(
		'type'    => 'content',
		'content' => sprintf(__('Get your credentials from %sBayarcash Console Profile%s', 'bayarcash-for-fluent-forms'), '<a href="https://console.bayar.cash/profile" target="_blank">', '</a>'),
	),
	array(
		'id'      => 'enable-sandbox',
		'type'    => 'switcher',
		'title'   => __('Enable Sandbox Mode', 'bayarcash-for-fluent-forms'),
		'desc'    => __('Toggle this to switch between sandbox and production environments.', 'bayarcash-for-fluent-forms'),
		'help'    => __('Use sandbox mode to test payments without processing real transactions. Make sure to use sandbox credentials when this is enabled.', 'bayarcash-for-fluent-forms'),
		'default' => false,
		'class'   => 'ff-bayarcash-sandbox-switcher'
	),
	array(
		'id'         => 'sandbox-notice',
		'type'       => 'notice',
		'style'      => 'warning',
		'content'    => __('Sandbox mode is enabled. Make sure to use sandbox credentials for testing.', 'bayarcash-for-fluent-forms'),
		'dependency' => array('enable-sandbox', '==', 'true'),
	),
	array(
		'id'    => 'personal-access-token',
		'type'  => 'textarea',
		'title' => __( 'Personal Access Token', 'bayarcash-for-fluent-forms' ),
		'desc'  => sprintf(__( 'Enter your Personal Access Token. Get it from %sBayarcash Console Profile%s', 'bayarcash-for-fluent-forms' ), '<a href="https://console.bayar.cash/profile" target="_blank">', '</a>'),
		'help'  => __( 'Personal Access Token is used to authenticate your requests to the Bayarcash API.', 'bayarcash-for-fluent-forms' ),
		'after' => '
            <div class="ff-bayarcash-token-status" style="margin-top: 10px;">
                <span class="status-message"></span>
                <button type="button" class="button button-secondary verify-token" style="margin-left: 10px;" onclick="verifyBayarcashToken()">
                    ' . __('Verify Token', 'bayarcash-for-fluent-forms') . '
                </button>
            </div>
            <script>
            let isTokenValid = false;

            function updatePortalList() {
                const portalSelect = document.querySelector("select[data-depend-id=\'portal-key\']");
                const portalField = portalSelect ? portalSelect.closest(".csf-field") : null;
                if (!portalSelect || !portalField) return;

                // Store current selected value
                const currentValue = portalSelect.value;

                const token = document.querySelector("textarea[data-depend-id=\'personal-access-token\']").value;
                const sandboxSwitch = document.querySelector(".ff-bayarcash-sandbox-switcher .csf--switcher");
                const isSandbox = sandboxSwitch ? sandboxSwitch.classList.contains("csf--active") : false;

                if (!token || !isTokenValid) {
                    // Clear and disable portal list if no token or invalid token
                    portalSelect.innerHTML = "<option value=\'\'>Please verify Personal Access Token first</option>";
                    portalSelect.disabled = true;
                    portalField.classList.add("csf-field-disabled");
                    return;
                }

                // Show loading state but maintain current value if exists
                if (currentValue) {
                    portalSelect.innerHTML = `
                        <option value=\'\'>${currentValue ? "Loading portals..." : "Select a Portal"}</option>
                        ${currentValue ? `<option value=\'${currentValue}\' selected>${currentValue}</option>` : ""}
                    `;
                } else {
                    portalSelect.innerHTML = "<option value=\'\'>Loading portals...</option>";
                }
                portalSelect.disabled = true;
                portalField.classList.add("csf-field-disabled");

                const api_url = isSandbox
                    ? "https://console.bayarcash-sandbox.com/api/v2/portals"
                    : "https://console.bayar.cash/api/v2/portals";

                jQuery.ajax({
                    url: api_url,
                    type: "GET",
                    headers: {
                        "Authorization": "Bearer " + token,
                        "Accept": "application/json"
                    },
                    success: function(response) {
                        let selectedFound = false;
                        portalSelect.innerHTML = "<option value=\'\'>Select a Portal</option>";
                        
                        if (response.data && Array.isArray(response.data)) {
                            response.data.forEach(function(portal) {
                                const option = document.createElement("option");
                                option.value = portal.portal_key;
                                option.textContent = portal.portal_name;
                                
                                // If this portal matches the previously selected value, select it
                                if (portal.portal_key === currentValue) {
                                    option.selected = true;
                                    selectedFound = true;
                                }
                                
                                portalSelect.appendChild(option);
                            });
                            
                            portalSelect.disabled = false;
                            portalField.classList.remove("csf-field-disabled");
                            
                            if (currentValue && !selectedFound) {
	                            const statusMessage = document.querySelector(".ff-bayarcash-token-status .status-message");
	                            if (statusMessage) {
		                            statusMessage.innerHTML += "<br>Warning: Previously selected portal is no longer available.";
		                            statusMessage.classList.add("warning");
	                            }
                            }
                        } else {
								portalSelect.innerHTML = "<option value=\'\'>No portals available</option>";
								portalSelect.disabled = true;
								portalField.classList.add("csf-field-disabled");
                        }
                    },
                    error: function() {
					if (currentValue) {
						portalSelect.innerHTML = `
				                                <option value=\'\'>Error loading portals</option>
				                                <option value=\'${currentValue}\' selected>${currentValue}</option>
				                            `;
					} else {
						portalSelect.innerHTML = "<option value=\'\'>Error loading portals</option>";
					}
					portalSelect.disabled = true;
					portalField.classList.add("csf-field-disabled");
				}
                });
            }

            function verifyBayarcashToken(manual = false) {
                const token = document.querySelector("textarea[data-depend-id=\'personal-access-token\']").value;
                const sandboxSwitch = document.querySelector(".ff-bayarcash-sandbox-switcher .csf--switcher");
                const isSandbox = sandboxSwitch ? sandboxSwitch.classList.contains("csf--active") : false;
                const statusMessage = document.querySelector(".ff-bayarcash-token-status .status-message");
                
                if (!token) {
                    statusMessage.innerHTML = "";
                    statusMessage.classList.remove("success", "error");
                    isTokenValid = false;
                    updatePortalList();
                    return;
                }
                
                statusMessage.innerHTML = "' . __('Verifying...', 'bayarcash-for-fluent-forms') . '";
                statusMessage.classList.remove("success", "error");
                
                jQuery.ajax({
                    url: ajaxurl,
                    type: "POST",
                    data: {
                        action: "verify_bayarcash_token",
                        token: token,
                        is_sandbox: isSandbox,
                        nonce: "' . wp_create_nonce('bayarcash_settings') . '"
                    },
                    success: function(response) {
                        statusMessage.innerHTML = response.message;
                        statusMessage.classList.remove("success", "error");
                        statusMessage.classList.add(response.status);
                        
                        isTokenValid = (response.status === "success");
                        updatePortalList();
                    },
                    error: function() {
                        statusMessage.innerHTML = "' . __('Verification failed', 'bayarcash-for-fluent-forms') . '";
                        statusMessage.classList.remove("success");
                        statusMessage.classList.add("error");
                        isTokenValid = false;
                        updatePortalList();
                    }
                });
            }
            
            // Verify on page load if token exists
            document.addEventListener("DOMContentLoaded", function() {
                const token = document.querySelector("textarea[data-depend-id=\'personal-access-token\']");
                if (token && token.value) {
                    verifyBayarcashToken();
                } else {
                    isTokenValid = false;
                    updatePortalList();
                }
                
                // When sandbox mode changes, re-verify if token exists
                const sandboxSwitch = document.querySelector(".ff-bayarcash-sandbox-switcher .csf--switcher");
                if (sandboxSwitch) {
                    // Create a MutationObserver to watch for class changes
                    const observer = new MutationObserver(function(mutations) {
                        mutations.forEach(function(mutation) {
                            if (mutation.attributeName === "class") {
                                const token = document.querySelector("textarea[data-depend-id=\'personal-access-token\']");
                                if (token && token.value) {
                                    verifyBayarcashToken();
                                } else {
                                    isTokenValid = false;
                                    updatePortalList();
                                }
                            }
                        });
                    });

                    // Start observing the sandbox switch for class changes
                    observer.observe(sandboxSwitch, {
                        attributes: true
                    });
                }

                // Monitor token changes
                if (token) {
                    token.addEventListener("input", function() {
                        if (!this.value.trim()) {
                            isTokenValid = false;
                            updatePortalList();
                        }
                    });
                }
            });
            
            // Add styles
            const style = document.createElement("style");
            style.textContent = `
                .ff-bayarcash-token-status .success { color: green; }
                .ff-bayarcash-token-status .error { color: red; }
                .ff-bayarcash-token-status .warning { color: orange; }
            `;
            document.head.appendChild(style);
            </script>
        ',
	),
	array(
		'id'         => 'portal-key',
		'type'       => 'select',
		'title'      => __( 'Portal Key', 'bayarcash-for-fluent-forms' ),
		'desc'       => __( 'Select your Portal from the list.', 'bayarcash-for-fluent-forms' ),
		'help'       => __( 'Portal Key is required for accessing the Bayarcash portal features.', 'bayarcash-for-fluent-forms' ),
		'options'    => 'get_bayarcash_portals',
		'class'      => 'ff-bayarcash-portal-select',
	),
	array(
		'id'    => 'api-secret-key',
		'type'  => 'text',
		'title' => __( 'API Secret Key', 'bayarcash-for-fluent-forms' ),
		'desc'  => sprintf(__( 'Enter your API Secret Key. Get it from %sBayarcash Console Profile%s', 'bayarcash-for-fluent-forms' ), '<a href="https://console.bayar.cash/profile" target="_blank">', '</a>'),
		'help'  => __( 'API Secret Key is used to secure your API requests to Bayarcash.', 'bayarcash-for-fluent-forms' ),
	),
	array(
		'type'    => 'subheading',
		'content' => 'Miscellaneous',
	),
	array(
		'id'          => 'payment-title',
		'type'        => 'text',
		'title'       => __( 'Payment Title', 'bayarcash-for-fluent-forms' ),
		'desc'        => __( 'Enter your Payment Title. Default is <strong>Bayarcash</strong>', 'bayarcash-for-fluent-forms' ),
		'help'        => __( 'This allows you to customize the payment title.', 'bayarcash-for-fluent-forms' ),
		'placeholder' => 'Bayarcash',
		'default'     => 'Bayarcash',
	),
);

// Create the settings sections
CSF_Setup::createOptions($slug, array(
	'framework_title' => sprintf(
		'<img src="%sassets/bayarcash.png" style="max-height: 40px; margin-right: 5px; vertical-align: middle;" /> %s',
		$plugin_url,
		__( 'for Fluent Forms', 'bayarcash-for-fluent-forms' )
	),
	'menu_title'  => __( 'Bayarcash Settings', 'bayarcash-for-fluent-forms' ),
	'menu_slug'   => 'bayarcash-for-fluent-forms',
	'menu_type'   => 'submenu',
	'menu_parent' => 'fluent_forms',
	'footer_text' => sprintf( __( 'Bayarcash for Fluent Forms %s', 'bayarcash-for-fluent-forms' ) , FF_BAYARCASH_MODULE_VERSION ),
	'theme'       => 'light',
));

// Create single section for Global Configuration
CSF_Setup::createSection($slug, array(
	'id'          => 'global-configuration',
	'title'       => __( 'Global Configuration', 'bayarcash-for-fluent-forms' ),
	'icon'        => 'fa fa-home',
	'fields'      => $global_configuration_fields
));