<?php
use FluentForm\App\Services\Form\SubmissionHandlerService;
use FluentForm\Framework\Helpers\ArrayHelper;
use FluentForm\App\Helpers\Helper;
use FluentForm\App\Services\FormBuilder\Notifications\EmailNotificationActions;
use FluentForm\App\Services\FormBuilder\ShortCodeParser;
use FluentFormPro\Payments\PaymentMethods\BaseProcessor;
use Webimpian\BayarcashSdk\Bayarcash;
use Webimpian\BayarcashSdk\Exceptions\ValidationException;
use FluentFormPro\Payments\PaymentHelper;

class Bayarcash_Fluent_Forms_Purchase extends BaseProcessor {
	private static $_instance;
	private Bayarcash $bayarcashSdk;
	private array $supported_currencies = ['MYR'];
	protected $method = 'bayarcash';
	private array $payment_methods = [
		'fpx_banking' => [
			'id' => 1,
			'title' => 'FPX Online Banking'
		],
		'credit_card' => [
			'id' => 4,
			'title' => 'Credit/Debit Card'
		],
		'duitnow_banking' => [
			'id' => 5,
			'title' => 'DuitNow Online Banking'
		],
		'duitnow_qr' => [
			'id' => 6,
			'title' => 'DuitNow QR'
		],
		'spaylater' => [
			'id' => 7,
			'title' => 'ShopeePayLater'
		],
		'boost_payflex' => [
			'id' => 8,
			'title' => 'Boost PayFlex'
		],
		'indo_banking' => [
			'id' => 9,
			'title' => 'Indonesia Online Banking'
		],
		'indo_ewallet' => [
			'id' => 10,
			'title' => 'Indonesia E-Wallet'
		],
		'nets_sg' => [
			'id' => 11,
			'title' => 'NETS Singapore'
		]
	];

	public static function get_instance(): Bayarcash_Fluent_Forms_Purchase {
		if (self::$_instance == null) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function __construct() {
		$this->add_action();
	}

	public function add_action(): void {
		foreach (array_keys($this->payment_methods) as $method) {
			add_action("fluentform/process_payment_{$method}", array($this, 'handlePaymentAction'), 10, 6);
			add_action("fluentform/payment_frameless_{$method}", array($this, 'handle_return_url'));
			add_action("fluentform/ipn_endpoint_{$method}", array($this, 'handle_return_url'));
		}
	}

	private function get_fee_settings_key($payment_method): string {
		// Payment methods that use their own settings
		$direct_methods = [
			'fpx_banking',
			'credit_card',
			'duitnow_banking'
		];
		return in_array($payment_method, $direct_methods) ? $payment_method : 'duitnow_qr';
	}

	private function calculate_fee($payment_method, $base_amount, $form_id = null): array {
		$options = get_option(FF_BAYARCASH_FSLUG);
		$postfix = $form_id ? "-$form_id" : '';

		// Get the appropriate settings key for the payment method
		$settings_key = $this->get_fee_settings_key($payment_method);

		// Initialize fee components
		$flat_fee = 0;
		$percentage_fee = 0;

		// Check if fees are enabled for this method
		if (!empty($options["{$settings_key}_fee_enable"])) {
			$fee_type = $options["{$settings_key}_fee_type"];

			// Calculate fees based on type
			if ($fee_type === 'flat' || $fee_type === 'both') {
				// Convert flat fee from RM to sen
				$flat_fee = round(floatval($options["{$settings_key}_flat_fee"]) * 100);
			}

			if ($fee_type === 'percentage' || $fee_type === 'both') {
				$percentage = floatval($options["{$settings_key}_percentage_fee"]);
				// Base amount is already in sen, so just calculate percentage directly
				$percentage_fee = round(($base_amount * $percentage) / 100);
			}
		}

		// Calculate total fee (already in sen)
		$fee_in_sen = $flat_fee + $percentage_fee;

//		error_log(sprintf(
//			'[Bayarcash FF] Fee Calculation Details - Method: %s, Settings: %s, Flat: %s sen, Percentage: %s sen (%.2f%%), Total: %s sen',
//			$payment_method,
//			$settings_key,
//			$flat_fee,
//			$percentage_fee,
//			$percentage,
//			$fee_in_sen
//		));

		return [
			'fee_amount' => $fee_in_sen,
			'total_with_fee' => $base_amount + $fee_in_sen,
			'settings_used' => $settings_key
		];
	}

	public function handlePaymentAction($submissionId, $submissionData, $form, $methodSettings, $hasSubscriptions, $totalPayable): void {
		$this->validate_if_subscription($hasSubscriptions);
		$this->setSubmissionId($submissionId);
		$this->form = $form;
		$submission = $this->getSubmission();
		$amountTotal = $this->getAmountTotal();

		$this->is_form_currency_supported(strtoupper($submission->currency));

		// Get the payment method from the current hook
		$payment_method = current_filter();
		$payment_method = str_replace('fluentform/process_payment_', '', $payment_method);

		$transactionId = $this->insertTransaction([
			'payment_total' => $amountTotal,
			'status' => 'pending',
			'currency' => strtoupper($submission->currency),
			'payment_method' => $this->payment_methods[$payment_method]['title'],
		]);

		$transaction = $this->getTransaction($transactionId);
		$this->create_purchase($transaction, $submission, $form, $methodSettings, $payment_method);
	}

	private function create_purchase($transaction, $submission, $form, $methodSettings, $payment_method): void {
		$option = $this->get_settings($form->id);

		// Log the selected payment method and its ID
		do_action('ff_log_data', [
			'parent_source_id' => $form->id,
			'source_type' => 'submission_item',
			'source_id' => $submission->id,
			'component' => 'Payment',
			'status' => 'info',
			'title' => __('Selected Payment Method', 'bayarcash-for-fluent-forms'),
			'description' => sprintf(
				__('Payment Method: %s, Payment Channel ID: %d', 'bayarcash-for-fluent-forms'),
				$payment_method,
				$this->payment_methods[$payment_method]
			),
		]);

		$bayarcash = new Bayarcash($option['personal_access_token']);

		// Set sandbox mode if enabled
		if ($option['sandbox_mode']) {
			$bayarcash->useSandbox();
		}

		// Calculate base amount in sen
		$amount_in_sen = $transaction->payment_total;

		// Calculate fees
		$fee_data = $this->calculate_fee($payment_method, $amount_in_sen, $form->id);

		// Log fee calculation
		do_action('ff_log_data', [
			'parent_source_id' => $form->id,
			'source_type' => 'submission_item',
			'source_id' => $submission->id,
			'component' => 'Payment',
			'status' => 'info',
			'title' => __('Fee Calculation Summary', 'bayarcash-for-fluent-forms'),
			'description' => sprintf(
				__('Base Amount: %d sen, Fee Amount: %d sen, Total: %d sen, Settings Used: %s',
					'bayarcash-for-fluent-forms'),
				$amount_in_sen,
				$fee_data['fee_amount'],
				$fee_data['total_with_fee'],
				$fee_data['settings_used']
			)
		]);

		// Format fee note
		$fee_note = sprintf(
			'Payment Processing Fee: RM %.2f (Settings: %s)',
			$fee_data['fee_amount'] / 100, // Convert sen to RM
			ucfirst(str_replace('_', ' ', $fee_data['settings_used'])) // Format settings string
		);

		// Store fee information in notes
		Helper::setSubmissionMeta(
			$submission->id,
			'_notes',
			$fee_note,
			$form->id
		);

		// Store fee amount in transaction meta
		$this->setMetaData('_payment_fee', $fee_data['fee_amount']);
		$this->setMetaData('_payment_fee_settings_used', $fee_data['settings_used']);

		// Prepare payment intent data with total including fee
		$final_amount = $fee_data['total_with_fee'] / 100; // Convert back to RM for API

		// Prepare payment intent data
		$paymentData = [
			'portal_key' => $option['portal_key'],
			'payment_channel' => $this->payment_methods[$payment_method]['id'],
			'order_number' => $submission->id,
			'amount' => $final_amount,
			'payer_name' => substr(PaymentHelper::getCustomerName($submission, $form), 0, 128),
			'payer_email' => PaymentHelper::getCustomerEmail($submission, $form),
			'payer_telephone_number' => '',
			'return_url' => add_query_arg([
				'fluentform_payment_api_notify' => 1,
				'payment_method' => $payment_method,
				'submission_id' => $submission->id
			], site_url('index.php')),
			'platform_id' => 'fluentforms'
		];

		// Log payment data being sent to Bayarcash
		do_action('ff_log_data', [
			'parent_source_id' => $form->id,
			'source_type' => 'submission_item',
			'source_id' => $submission->id,
			'component' => 'Payment',
			'status' => 'info',
			'title' => __('Payment Data', 'bayarcash-for-fluent-forms'),
			'description' => sprintf(
				__('Payment Data being sent to Bayarcash: %s', 'bayarcash-for-fluent-forms'),
				print_r($paymentData, true)
			),
		]);

		// Create checksum for payment intent
		$paymentData['checksum'] = $bayarcash->createPaymentIntenChecksumValue($option['secret_key'], $paymentData);

		try {
			// Create payment intent
			$response = $bayarcash->createPaymentIntent($paymentData);

			// Parse the response to get the payment URL
			if (is_object($response) && isset($response->url)) {
				$payment_url = $response->url;
			} else {
				throw new \Exception(__('Invalid response from payment gateway', 'bayarcash-for-fluent-forms'));
			}

			// Update transaction details
			$this->updateTransaction($transaction->id, [
				'payment_mode' => $option['sandbox_mode'] ? 'test' : 'live',
				'charge_id' => $response->id ?? '',
			]);

			$this->setMetaData('_bayarcash_purchase_id', $response->id ?? '' );

			// Log redirect
			do_action('ff_log_data', [
				'parent_source_id' => $form->id,
				'source_type' => 'submission_item',
				'source_id' => $submission->id,
				'component' => 'Payment',
				'status' => 'info',
				'title' => __('Redirect to Bayarcash', 'bayarcash-for-fluent-forms'),
				'description' => sprintf(__('User redirect to Bayarcash for completing the payment: %s', 'bayarcash-for-fluent-forms'), esc_url($payment_url)),
			]);

			// Send JSON response with redirect URL
			wp_send_json_success([
				'nextAction' => 'payment',
				'actionName' => 'normalRedirect',
				'redirect_url' => esc_url($payment_url),
				'message' => __('You are redirecting to Bayarcash to complete the purchase. Please wait while you are redirecting....', 'bayarcash-for-fluent-forms'),
				'result' => [
					'insert_id' => $submission->id
				]
			], 200);

		} catch (\Exception $e) {
			do_action('ff_log_data', [
				'parent_source_id' => $form->id,
				'source_type' => 'submission_item',
				'source_id' => $submission->id,
				'component' => 'Payment',
				'status' => 'error',
				'title' => __('Payment Error', 'bayarcash-for-fluent-forms'),
				'description' => $e->getMessage(),
			]);

			wp_send_json_success([
				'message' => $e->getMessage()
			], 500);
		}
	}

	public function handleSessionRedirectBack($data) {
		$submissionId = intval($data['fluentform_payment']);
		$this->setSubmissionId($submissionId);

		$submission = $this->getSubmission();
		$transactionHash = sanitize_text_field($data['transaction_hash']);
		$transaction = $this->getTransaction($transactionHash, 'transaction_hash');

		if (!$transaction || !$submission) {
			return;
		}

		$type = $transaction->status;
		$this->getForm();

		if ($type == 'paid') {
			$returnData = $this->getReturnData();
		} elseif ($type == 'pending') {
			$returnData = [
				'insert_id' => $submission->id,
				'title' => __('Payment Pending', 'bayarcash-for-fluent-forms'),
				'result' => false,
				'error' => __('Your payment is being processed. Please wait while we confirm your payment.', 'bayarcash-for-fluent-forms'),
				'payment_status' => 'pending'
			];
		} else {
			$returnData = [
				'insert_id' => $submission->id,
				'title' => __('Payment Cancelled', 'bayarcash-for-fluent-forms'),
				'result' => false,
				'error' => __('Looks like you have cancelled the payment', 'bayarcash-for-fluent-forms')
			];
		}

		$returnData['type'] = 'success';
		$returnData['is_new'] = false;

		// Log the payment status
		do_action('ff_log_data', [
			'parent_source_id' => $this->form->id,
			'source_type' => 'submission_item',
			'source_id' => $submission->id,
			'component' => 'Payment',
			'status' => 'info',
			'title' => __('Payment Return Status', 'bayarcash-for-fluent-forms'),
			'description' => sprintf(
				__('Payment status: %s for submission ID: %s', 'bayarcash-for-fluent-forms'),
				$type,
				$submission->id
			)
		]);

		$this->showPaymentView($returnData);
	}

	public function handlePaid($submission, $transaction, $vendorTransaction) {
		$this->setSubmissionId($submission->id);

		if ($this->getMetaData('is_form_action_fired') == 'yes') {
			return $this->completePaymentSubmission(false);
		}

		$status = sanitize_text_field($vendorTransaction['status']);

		$updateData = apply_filters('ff_chip_handle_paid_data', [
			'payment_note' => maybe_serialize($vendorTransaction),
			'charge_id' => sanitize_text_field($vendorTransaction['id']),
			'payer_email' => $vendorTransaction['client']['email'],
			'payment_total' => intval($vendorTransaction['purchase']['total']),
		], $submission, $transaction, $vendorTransaction);

		$this->updateTransaction($transaction->id, $updateData);
		$this->changeSubmissionPaymentStatus($status);
		$this->changeTransactionStatus($transaction->id, $status);
		$this->recalculatePaidTotal();
		$this->setMetaData('is_form_action_fired', 'yes');

		$submission_service = new SubmissionHandlerService();
		$submission_service->processSubmissionData($this->submissionId, $submission->response, $this->getForm());

		$email_feeds = wpFluent()->table('fluentform_form_meta')
		                         ->where('form_id', $this->getForm()->id)
		                         ->where('meta_key', 'notifications')
		                         ->get();

		if (!$email_feeds) {
			return;
		}

		$form_data = $submission->response;
		$notification_manager = new \FluentForm\App\Services\Integrations\GlobalNotificationManager(wpFluentForm());

		$active_email_feeds = $notification_manager->getEnabledFeeds($email_feeds, $form_data, $submission->id);

		if (!$active_email_feeds) {
			return;
		}

		$after_success_email_feeds = array_filter($active_email_feeds, function ($feed) {
			return 'payment_success' == ArrayHelper::get($feed, 'settings.feed_trigger_event');
		});

		if (!$after_success_email_feeds || 'yes' === Helper::getSubmissionMeta($submission->id, '_ff_chip_on_payment_success')) {
			return;
		}

		$ena = new EmailNotificationActions(wpFluentForm());
		$entry = $ena->getEntry($submission->id);

		foreach ($after_success_email_feeds as $feed) {
			$processedValues = $feed['settings'];
			unset($processedValues['conditionals']);

			$processedValues = ShortCodeParser::parse(
				$processedValues,
				$submission->id,
				$form_data,
				$this->getForm(),
				false,
				$feed['meta_key']
			);
			$feed['processedValues'] = $processedValues;
		}

		Helper::setSubmissionMeta($submission->id, '_ff_chip_on_payment_success', 'yes', $this->getForm()->id);
	}


	public function handleFailed($submission, $transaction, $vendorTransaction): void {
		$this->setSubmissionId($submission->id);

		$status = 'failed';

		$updateData = [
			'payment_note' => maybe_serialize($vendorTransaction),
		];

		$this->updateTransaction($transaction->id, $updateData);
		$this->changeSubmissionPaymentStatus($status);
		$this->changeTransactionStatus($transaction->id, $status);
	}

	public function handle_return_url() {
		// Verify request method
		if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
			wp_die('Invalid request method');
		}

		$submission_id = isset($_POST['order_number']) ? absint($_POST['order_number']) : 0;

		// Set submission ID early to get form data for logging
		$this->setSubmissionId($submission_id);
		$submission = $this->getSubmission();
		$form = $this->getForm();

		// Get raw POST data and log it for debugging
		do_action('ff_log_data', [
			'parent_source_id' => $form->id,
			'source_type' => 'submission_item',
			'source_id' => $submission_id,
			'component' => 'Payment',
			'status' => 'info',
			'title' => __('Bayarcash Webhook Raw Data', 'bayarcash-for-fluent-forms'),
			'description' => print_r($_POST, true)
		]);

		// Get and sanitize POST data
		$data = [
			'record_type' => sanitize_text_field($_POST['record_type'] ?? ''),
			'transaction_id' => sanitize_text_field($_POST['transaction_id'] ?? ''),
			'exchange_reference_number' => sanitize_text_field($_POST['exchange_reference_number'] ?? ''),
			'exchange_transaction_id' => sanitize_text_field($_POST['exchange_transaction_id'] ?? ''),
			'order_number' => $submission_id,
			'currency' => sanitize_text_field($_POST['currency'] ?? ''),
			'amount' => floatval($_POST['amount'] ?? 0),
			'payer_name' => sanitize_text_field($_POST['payer_name'] ?? ''),
			'payer_email' => sanitize_email($_POST['payer_email'] ?? ''),
			'payer_bank_name' => sanitize_text_field($_POST['payer_bank_name'] ?? ''),
			'status' => isset($_POST['status']) ? absint($_POST['status']) : null,
			'status_description' => sanitize_text_field($_POST['status_description'] ?? ''),
			'datetime' => sanitize_text_field($_POST['datetime'] ?? ''),
			'payer_telephone_number' => sanitize_text_field($_POST['payer_telephone_number'] ?? '')
		];

		// Validate required fields
		$required_fields = ['transaction_id', 'order_number', 'amount', 'status'];
		foreach ($required_fields as $field) {
			if (!isset($data[$field])) {
				do_action('ff_log_data', [
					'parent_source_id' => $form->id,
					'source_type' => 'submission_item',
					'source_id' => $submission_id,
					'component' => 'Payment',
					'status' => 'error',
					'title' => __('Bayarcash Webhook Validation Error', 'bayarcash-for-fluent-forms'),
					'description' => sprintf(__('Missing required field: %s', 'bayarcash-for-fluent-forms'), $field)
				]);
				wp_die('Missing required field: ' . $field);
			}
		}

		// Get submission and transaction
		if (!$submission) {
			wp_die('Invalid submission');
		}

		$transaction = $this->getTransaction($submission_id, 'submission_id');
		if (!$transaction) {
			wp_die('Transaction not found');
		}

		$meta_value = sprintf(
			'Transaction ID: %s | Amount: %s %s | Status: %s | Payer: %s (%s) | Bank: %s | Date: %s | Order: %s | Ref: %s | Exchange ID: %s',
			$data['transaction_id'],
			$data['amount'],
			$data['currency'],
			$data['status_description'],
			$data['payer_name'],
			$data['payer_email'],
			$data['payer_bank_name'],
			$data['datetime'],
			$data['order_number'],
			$data['exchange_reference_number'],
			$data['exchange_transaction_id']
		);

		Helper::setSubmissionMeta(
			$submission_id,
			'_notes',
			$meta_value,
			$form->id
		);

		// Determine payment status based on webhook status
		$payment_status = 'pending';  // Default status
		if ($data['status'] === 3) {
			$payment_status = 'paid';
		} elseif ($data['status'] > 0) {  // Only mark as failed if status is greater than 0
			$payment_status = 'failed';
		}

		// Log the status determination
		do_action('ff_log_data', [
			'parent_source_id' => $form->id,
			'source_type' => 'submission_item',
			'source_id' => $submission_id,
			'component' => 'Payment',
			'status' => 'info',
			'title' => __('Bayarcash Payment Status Determination', 'bayarcash-for-fluent-forms'),
			'description' => sprintf(
				__('Webhook status: %d, Determined payment status: %s', 'bayarcash-for-fluent-forms'),
				$data['status'],
				$payment_status
			)
		]);

		// Prepare payment data from POST
		$payment = [
			'status' => $payment_status,
			'id' => $data['transaction_id'],
			'client' => [
				'email' => $data['payer_email']
			],
			'purchase' => [
				'total' => $data['amount'] * 100 // Convert to cents
			]
		];

		// Process payment based on status
		if ($payment_status === 'paid') {
			$this->handlePaid($submission, $transaction, $payment);
		} elseif ($payment_status === 'failed') {
			$this->handleFailed($submission, $transaction, $payment);
		} else {
			// For pending status, update the transaction but don't change the status
			$updateData = [
				'payment_note' => maybe_serialize($payment),
			];
			$this->updateTransaction($transaction->id, $updateData);

			do_action('ff_log_data', [
				'parent_source_id' => $form->id,
				'source_type' => 'submission_item',
				'source_id' => $submission_id,
				'component' => 'Payment',
				'status' => 'info',
				'title' => __('Payment Status Maintained as Pending', 'bayarcash-for-fluent-forms'),
				'description' => sprintf(
					__('Transaction ID: %s maintained pending status for status code 0', 'bayarcash-for-fluent-forms'),
					$data['transaction_id']
				)
			]);
		}

		// Prepare data for session redirect
		$redirectData = [
			'fluentform_payment' => $submission_id,
			'payment_method' => 'bayarcash',
			'transaction_hash' => $transaction->transaction_hash,
			'type' => $payment_status === 'paid' ? 'success' : ($payment_status === 'failed' ? 'failed' : 'pending')
		];

		// Handle session redirect
		$this->handleSessionRedirectBack($redirectData);

		// Redirect to payment handler
		wp_redirect(add_query_arg($redirectData, site_url('index.php')));
		exit;
	}

	private function is_form_currency_supported($currency): void {
		if (!in_array($currency, $this->supported_currencies)) {
			echo sprintf(__('Error! Currency not supported. The only supported currency is MYR and the current currency is %s.', 'bayarcash-for-fluent-forms'), esc_html($currency));
			exit(200);
		}
	}

	private function get_settings($form_id): array {
		$options = get_option(FF_BAYARCASH_FSLUG);
		$postfix = '';
		$form_cid = 'form-customize-' . $form_id;

		// Check if form-specific customization is enabled
		if (array_key_exists($form_cid, $options) && $options[$form_cid]) {
			$postfix = "-$form_id";
		}

		return array(
			// Credentials
			'sandbox_mode' => empty($options['enable-sandbox' . $postfix]) ? false : $options['enable-sandbox' . $postfix],
			'personal_access_token' => $options['personal-access-token' . $postfix],
			'portal_key' => $options['portal-key' . $postfix],
			'secret_key' => $options['api-secret-key' . $postfix],

			// Payment Settings
			'payment_title' => empty($options['payment-title' . $postfix]) ? 'Bayarcash' : $options['payment-title' . $postfix]
		);
	}

	private function validate_if_subscription($has_subscription): void {
		if ($has_subscription) {
			wp_send_json([
				'errors' => __('Error: Bayarcash does not support subscriptions right now.', 'bayarcash-for-fluent-forms')
			], 423);
		}
	}
}

Bayarcash_Fluent_Forms_Purchase::get_instance();