(function($) {
    'use strict';

    let isTokenValid = false;

    function updatePortalList() {
        const portalSelect = document.querySelector("select[data-depend-id='portal-key']");
        const portalField = portalSelect ? portalSelect.closest(".csf-field") : null;
        if (!portalSelect || !portalField) return;

        // Store current selected value
        const currentValue = portalSelect.value;

        const token = document.querySelector("textarea[data-depend-id='personal-access-token']").value;
        const sandboxSwitch = document.querySelector(".ff-bayarcash-sandbox-switcher .csf--switcher");
        const isSandbox = sandboxSwitch ? sandboxSwitch.classList.contains("csf--active") : false;

        if (!token || !isTokenValid) {
            portalSelect.innerHTML = "<option value=''>Please verify Personal Access Token first</option>";
            portalSelect.disabled = true;
            portalField.classList.add("csf-field-disabled");
            return;
        }

        if (currentValue) {
            portalSelect.innerHTML = `
                <option value=''>${currentValue ? "Loading portals..." : "Select a Portal"}</option>
                ${currentValue ? `<option value='${currentValue}' selected>${currentValue}</option>` : ""}
            `;
        } else {
            portalSelect.innerHTML = "<option value=''>Loading portals...</option>";
        }
        portalSelect.disabled = true;
        portalField.classList.add("csf-field-disabled");

        const api_url = isSandbox
            ? "https://console.bayarcash-sandbox.com/api/v2/portals"
            : "https://console.bayar.cash/api/v2/portals";

        $.ajax({
            url: api_url,
            type: "GET",
            headers: {
                "Authorization": "Bearer " + token,
                "Accept": "application/json"
            },
            success: function(response) {
                let selectedFound = false;
                portalSelect.innerHTML = "<option value=''>Select a Portal</option>";

                if (response.data && Array.isArray(response.data)) {
                    response.data.forEach(function(portal) {
                        const option = document.createElement("option");
                        option.value = portal.portal_key;
                        option.textContent = portal.portal_name;

                        if (portal.portal_key === currentValue) {
                            option.selected = true;
                            selectedFound = true;
                        }

                        portalSelect.appendChild(option);
                    });

                    portalSelect.disabled = false;
                    portalField.classList.remove("csf-field-disabled");

                    if (currentValue && !selectedFound) {
                        const statusMessage = document.querySelector(".ff-bayarcash-token-status .status-message");
                        if (statusMessage) {
                            statusMessage.innerHTML += "<br>Warning: Previously selected portal is no longer available.";
                            statusMessage.classList.add("warning");
                        }
                    }
                } else {
                    portalSelect.innerHTML = "<option value=''>No portals available</option>";
                    portalSelect.disabled = true;
                    portalField.classList.add("csf-field-disabled");
                }
            },
            error: function() {
                if (currentValue) {
                    portalSelect.innerHTML = `
                        <option value=''>Error loading portals</option>
                        <option value='${currentValue}' selected>${currentValue}</option>
                    `;
                } else {
                    portalSelect.innerHTML = "<option value=''>Error loading portals</option>";
                }
                portalSelect.disabled = true;
                portalField.classList.add("csf-field-disabled");
            }
        });
    }

    function verifyBayarcashToken(manual = false) {
        const token = document.querySelector("textarea[data-depend-id='personal-access-token']").value;
        const sandboxSwitch = document.querySelector(".ff-bayarcash-sandbox-switcher .csf--switcher");
        const isSandbox = sandboxSwitch ? sandboxSwitch.classList.contains("csf--active") : false;
        const statusMessage = document.querySelector(".ff-bayarcash-token-status .status-message");

        if (!token) {
            statusMessage.innerHTML = "";
            statusMessage.classList.remove("success", "error");
            isTokenValid = false;
            updatePortalList();
            return;
        }

        statusMessage.innerHTML = wp_bayarcash.verifying_text;
        statusMessage.classList.remove("success", "error");

        $.ajax({
            url: ajaxurl,
            type: "POST",
            data: {
                action: "verify_bayarcash_token",
                token: token,
                is_sandbox: isSandbox,
                nonce: wp_bayarcash.nonce
            },
            success: function(response) {
                statusMessage.innerHTML = response.message;
                statusMessage.classList.remove("success", "error");
                statusMessage.classList.add(response.status);

                isTokenValid = (response.status === "success");
                updatePortalList();
            },
            error: function() {
                statusMessage.innerHTML = wp_bayarcash.verification_failed_text;
                statusMessage.classList.remove("success");
                statusMessage.classList.add("error");
                isTokenValid = false;
                updatePortalList();
            }
        });
    }

    // Initialize when document is ready
    $(document).ready(function() {
        const token = document.querySelector("textarea[data-depend-id='personal-access-token']");
        if (token && token.value) {
            verifyBayarcashToken();
        } else {
            isTokenValid = false;
            updatePortalList();
        }

        // Monitor sandbox mode changes
        const sandboxSwitch = document.querySelector(".ff-bayarcash-sandbox-switcher .csf--switcher");
        if (sandboxSwitch) {
            const observer = new MutationObserver(function(mutations) {
                mutations.forEach(function(mutation) {
                    if (mutation.attributeName === "class") {
                        const token = document.querySelector("textarea[data-depend-id='personal-access-token']");
                        if (token && token.value) {
                            verifyBayarcashToken();
                        } else {
                            isTokenValid = false;
                            updatePortalList();
                        }
                    }
                });
            });

            observer.observe(sandboxSwitch, {
                attributes: true
            });
        }

        // Monitor token changes
        if (token) {
            token.addEventListener("input", function() {
                if (!this.value.trim()) {
                    isTokenValid = false;
                    updatePortalList();
                }
            });
        }
    });

    // Add styles
    const style = document.createElement("style");
    style.textContent = `
        .ff-bayarcash-token-status .success { color: green; }
        .ff-bayarcash-token-status .error { color: red; }
        .ff-bayarcash-token-status .warning { color: orange; }
    `;
    document.head.appendChild(style);

    // Expose verifyBayarcashToken to global scope for the button onclick
    window.verifyBayarcashToken = verifyBayarcashToken;

})(jQuery);