(function($) {
    'use strict';

    class BayarcashFluentForms {
        constructor() {
            this.feeSettings = {};
            this.isUpdating = false;
            this.updateCount = 0;
            this.lastUpdateTime = Date.now();
            if (this.hasFluentForms()) {
                this.init();
            } else {
                console.log('No Fluent Forms detected, initialization skipped');
            }
        }

        hasFluentForms() {
            const forms = $('.fluentform');
            return forms.length > 0;
        }

        init() {
            this.feeSettings = window.bayarcashFF?.feeSettings || {};
            console.log('Init Fee Settings:', this.feeSettings);

            $(document).ready(() => {
                this.checkForForms();
            });

            $(document).on('fluentform_init_form_entries', (e, formData) => {
                if (formData && formData.$form) {
                    const formId = this.getFormIdFromElement(formData.$form);
                    if (formId) {
                        this.initializePaymentForm(formData.$form, formId);
                    }
                }
            });
        }

        getFormIdFromElement($element) {
            const wrapperClass = $element.closest('[class*="fluentform_wrapper_"]').attr('class');
            if (wrapperClass) {
                const matches = wrapperClass.match(/fluentform_wrapper_(\d+)/);
                return matches && matches[1] ? matches[1] : null;
            }
            return null;
        }

        checkForForms() {
            const forms = $('.fluentform');
            const hasFluentForms = forms.length > 0;
            console.log('Fluent Forms detected:', hasFluentForms);
            forms.each((_, form) => {
                const $form = $(form);
                const formId = this.getFormIdFromElement($form);
                const hasPayment = $form.find('.ff_payment_method').length > 0;

                if (formId && hasPayment) {
                    this.initializePaymentForm($form, formId);
                }
            });
        }

        initializePaymentForm($form, formId) {
            // Listen for payment method changes
            $form.find('.ff_payment_method').on('change', () => {
                this.updateGatewayFeeInput($form);
                this.updateTotals($form);
            });

            // Watch for changes in any input with data-payment_item="yes"
            $form.find('input[data-payment_item="yes"]').not('.bayarcash_fee').on('change', () => {
                this.updateGatewayFeeInput($form);
                this.updateTotals($form);
            });

            const $paymentSummary = $form.find('.ff_payment_summary');
            if ($paymentSummary.length) {
                this.observeTableChanges($form);
            }

            // Initial update
            setTimeout(() => {
                this.updateGatewayFeeInput($form);
                this.updateTotals($form);
            }, 100);
        }

        getCurrentPaymentMethod($form) {
            const selectedRadio = $form.find('.ff_payment_method:checked');
            return selectedRadio.length ? selectedRadio.val() : '';
        }

        parseAmount(amountText) {
            if (!amountText) return 0;
            // Remove 'RM' prefix, any commas, and spaces, then parse as float
            return parseFloat(amountText.replace('RM', '').replace(/,/g, '').trim()) || 0;
        }

        formatNumber(number) {
            return number.toFixed(2).replace(/\B(?=(\d{3})+(?!\d))/g, ",");
        }

        updateGatewayFeeInput($form) {
            const paymentMethod = this.getCurrentPaymentMethod($form);
            const originalTotal = this.calculateOriginalTotal($form);
            let gatewayFee = 0;
            let feeTitle = '';
            let isFeeEnabled = false;

            const formId = this.getFormIdFromElement($form);
            if (!formId) return;

            const feeConfig = this.getFeeConfig(paymentMethod);
            if (paymentMethod && feeConfig) {
                isFeeEnabled = true;
                gatewayFee = this.calculateFee(originalTotal, paymentMethod);
                feeTitle = this.getFeeDescription(paymentMethod);
            }

            const $feeWrapper = $form.find(`.bayarcash_${formId}`);
            if ($feeWrapper.length) {
                if (isFeeEnabled && gatewayFee > 0) {
                    $feeWrapper.show();
                    $feeWrapper.find('.ff-el-fee-prefix.bc-gateway').text(feeTitle);

                    const $feeInput = $feeWrapper.find('input[name="bayarcash_fee"]');
                    if ($feeInput.length) {
                        $feeInput.val('RM ' + gatewayFee.toFixed(2));
                        $feeInput.attr('data-fee', gatewayFee.toFixed(2));
                        $feeInput[0].dispatchEvent(new Event('change', { bubbles: true }));
                    }
                } else {
                    $feeWrapper.hide();
                    const $feeInput = $feeWrapper.find('input[name="bayarcash_fee"]');
                    if ($feeInput.length) {
                        $feeInput.val('0.00');
                        $feeInput.attr('data-fee', '0.00');
                        $feeInput[0].dispatchEvent(new Event('change', { bubbles: true }));
                    }
                }
            }

            return gatewayFee;
        }

        calculateOriginalTotal($form) {
            let originalTotal = 0;
            const $tableBody = $form.find('.ff_payment_summary table tbody');

            if (!$tableBody.length) {
                // If no payment summary table, look for payment inputs
                const $amountInputs = $form.find('input[data-payment_item="yes"]').not('.bayarcash_fee');
                $amountInputs.each((_, input) => {
                    const amount = this.parseAmount($(input).val());
                    originalTotal += amount;
                });
            } else {
                // If payment summary exists, calculate from table
                $tableBody.find('tr').each((_, row) => {
                    const $row = $(row);
                    const firstCell = $row.find('td:first').text();
                    if (!firstCell.includes('Fee')) {
                        const amountText = $row.find('td:last').text();
                        const amount = this.parseAmount(amountText);
                        originalTotal += amount;
                    }
                });
            }

            return originalTotal;
        }

        getFeeConfig(paymentMethod) {
            const ownSettingMethods = ['fpx_banking', 'credit_card', 'duitnow_banking'];
            const duitnowQRMethods = [
                'duitnow_qr',
                'spaylater',
                'boost_payflex',
                'indo_banking',
                'indo_ewallet',
                'nets_sg'
            ];

            const effectiveMethod = ownSettingMethods.includes(paymentMethod)
                ? paymentMethod
                : (duitnowQRMethods.includes(paymentMethod) ? 'duitnow_qr' : paymentMethod);

            const methodSettings = this.feeSettings[effectiveMethod] || {};
            const isEnabled = methodSettings[`${effectiveMethod}_fee_enable`] === 'true';

            if (!isEnabled) return null;

            return {
                enabled: true,
                feeType: methodSettings[`${effectiveMethod}_fee_type`] || 'flat',
                flatFee: parseFloat(methodSettings[`${effectiveMethod}_flat_fee`]) || 0,
                percentageFee: parseFloat(methodSettings[`${effectiveMethod}_percentage_fee`]) || 0,
                description: methodSettings[`${effectiveMethod}_fee_description`] || 'Processing Fee'
            };
        }

        calculateFee(amount, paymentMethod) {
            const config = this.getFeeConfig(paymentMethod);
            if (!config) return 0;

            let fee = 0;
            switch (config.feeType) {
                case 'flat':
                    fee = config.flatFee;
                    break;
                case 'percentage':
                    fee = amount * (config.percentageFee / 100);
                    break;
                case 'both':
                    fee = config.flatFee + (amount * (config.percentageFee / 100));
                    break;
            }

            return parseFloat(fee.toFixed(2));
        }

        getFeeDescription(paymentMethod) {
            const config = this.getFeeConfig(paymentMethod);
            if (!config) return '';

            let description = config.description;

            if (config.feeType === 'percentage') {
                const percentage = config.percentageFee % 1 === 0 ?
                    Math.floor(config.percentageFee) :
                    config.percentageFee.toFixed(1);
                description += ` (${percentage}%)`;
            } else if (config.feeType === 'both') {
                const percentage = config.percentageFee % 1 === 0 ?
                    Math.floor(config.percentageFee) :
                    config.percentageFee.toFixed(1);
                description += ` (RM${config.flatFee.toFixed(2)} + ${percentage}%)`;
            } else {
                description += ` (RM${config.flatFee.toFixed(2)})`;
            }

            return description;
        }

        updateTotals($form) {
            if (this.isUpdating) return;

            this.isUpdating = true;
            try {
                const originalTotal = this.calculateOriginalTotal($form);
                const gatewayFee = this.updateGatewayFeeInput($form);
                const total = originalTotal + gatewayFee;

                // Update payment summary table if it exists
                const $tableBody = $form.find('.ff_payment_summary table tbody');
                if ($tableBody.length) {
                    // Remove existing fee rows
                    $tableBody.find('tr').each((_, row) => {
                        const $row = $(row);
                        const firstCell = $row.find('td:first').text();
                        if (firstCell.includes('Fee')) {
                            $row.remove();
                        }
                    });

                    // Add new fee row if applicable
                    const paymentMethod = this.getCurrentPaymentMethod($form);
                    if (paymentMethod && gatewayFee > 0) {
                        const feeDescription = this.getFeeDescription(paymentMethod);
                        const $newRow = $(`
                            <tr>
                                <td>${feeDescription}</td>
                                <td>RM${this.formatNumber(gatewayFee)}</td>
                                <td>1</td>
                                <td>RM${this.formatNumber(gatewayFee)}</td>
                            </tr>
                        `);
                        $tableBody.append($newRow);
                    }

                    // Update table footer total
                    const $totalCell = $form.find('.ff_payment_summary table tfoot th:last');
                    if ($totalCell.length) {
                        $totalCell.text(`RM${this.formatNumber(total)}`);
                    }
                }

                // Update all instances of ff_order_total
                const $orderTotalElements = $form.find('.ff_order_total');
                if ($orderTotalElements.length) {
                    $orderTotalElements.each((_, element) => {
                        $(element).text(`RM${this.formatNumber(total)}`);
                    });
                }

            } catch (error) {
                console.error('Update totals error:', error);
            } finally {
                this.isUpdating = false;
            }
        }

        observeTableChanges($form) {
            const $paymentSummary = $form.find('.ff_payment_summary');
            if (!$paymentSummary.length) return;

            let updateTimeout;
            let observerPaused = false;

            const observer = new MutationObserver(() => {
                if (observerPaused) return;

                clearTimeout(updateTimeout);
                updateTimeout = setTimeout(() => {
                    if (!this.isUpdating) {
                        observerPaused = true;
                        this.updateGatewayFeeInput($form);
                        this.updateTotals($form);
                        setTimeout(() => {
                            observerPaused = false;
                        }, 100);
                    }
                }, 200);
            });

            observer.observe($paymentSummary[0], {
                childList: true,
                subtree: true,
                characterData: true
            });
        }
    }

    // Initialize when document is ready
    $(document).ready(() => {
        new BayarcashFluentForms();
    });

})(jQuery);